@@ -1,19 +1,19 @@
 import collections
 import dataclasses
 import functools
-import json
 import logging
 import os
 import re
 import shlex
 import subprocess
-from typing import Any, Callable, Iterable, Iterator, List, Mapping, Sequence, TypeVar, Union
+from typing import Any, Callable, Iterable, Iterator, List, Mapping, Sequence, TypeVar, Union, cast
 
 import mkdocs.exceptions
 import mkdocs.utils
 from cached_property import cached_property
 from mkdocstrings.handlers.base import BaseCollector, CollectionError
 
+from . import inventory
 from .items import DocConstant, DocItem, DocLocation, DocMapping, DocMethod, DocModule, DocType
 
 try:
@@ -29,7 +29,7 @@ D = TypeVar("D", bound=DocItem)
 
 class CrystalCollector(BaseCollector):
     def __init__(
-        self, crystal_docs_flags: Sequence[str] = (), source_locations: Mapping[str, str] = ()
+        self, crystal_docs_flags: Sequence[str] = (), source_locations: Mapping[str, str] = {}
     ):
         """Create a "collector", reading docs from `crystal doc` in the current directory.
 
@@ -48,11 +48,10 @@ class CrystalCollector(BaseCollector):
         ]
         if source_locations:
             command.append("--source-refname=master")
-        command += crystal_docs_flags
+        command += (s.format_map(_crystal_info) for s in crystal_docs_flags)
         log.debug("Running `%s`", " ".join(shlex.quote(arg) for arg in command))
 
         self._proc = subprocess.Popen(command, stdout=subprocess.PIPE)
-        self._collected = collections.Counter()
 
         # For unambiguous prefix match: add trailing slash, sort by longest path first.
         self._source_locations = sorted(
@@ -69,9 +68,8 @@ class CrystalCollector(BaseCollector):
         """The top-level namespace, represented as a fake module."""
         try:
             with self._proc:
-                data = json.load(self._proc.stdout)
-            data["program"]["full_name"] = ""
-            root = DocRoot(data["program"], None, None)
+                root = inventory.read(self._proc.stdout)
+            root.__class__ = DocRoot
             root.source_locations = self._source_locations
             return root
         finally:
@@ -96,29 +94,7 @@ class CrystalCollector(BaseCollector):
         item = self.root
         if identifier != "::":
             item = item.lookup(identifier)
-        view = DocView(item, config)
-        if isinstance(item, DocType):
-            self._collected.update((item,))
-            self._collected.update(view.walk_types())
-        else:
-            self._collected.setdefault(item.parent, 0)
-
-        return view
-
-    def teardown(self):
-        if log.isEnabledFor(logging.DEBUG):
-            mul_collected = ", ".join(
-                typ.abs_id for typ in self.root.walk_types() if self._collected[typ] > 1
-            )
-            if mul_collected:
-                log.debug(
-                    f"These types were put into the documentation more than once: {mul_collected}"
-                )
-            not_collected = ", ".join(
-                typ.abs_id for typ in self.root.walk_types() if typ not in self._collected
-            )
-            if not_collected:
-                log.debug(f"These types were never put into the documentation: {not_collected}")
+        return DocView(item, config)
 
 
 @dataclasses.dataclass
@@ -129,18 +105,12 @@ class _SourceDestination:
     def substitute(self, location: DocLocation) -> str:
         data = {"file": location.filename[len(self.src_path) :], "line": location.line}
         try:
-            return self.dest_url.format_map(collections.ChainMap(data, self))  # type: ignore
+            return self.dest_url.format_map(collections.ChainMap(data, _DictAccess(self), _crystal_info))  # type: ignore
         except KeyError as e:
             raise PluginError(
                 f"The source_locations template {self.dest_url!r} did not resolve correctly: {e}"
             )
 
-    def __getitem__(self, key):
-        try:
-            return getattr(self, key)
-        except AttributeError as e:
-            raise KeyError(str(e))
-
     @property
     def shard_version(self):
         return self._shard_version(os.path.dirname(self.src_path))
@@ -148,16 +118,51 @@ class _SourceDestination:
     @classmethod
     @functools.lru_cache(maxsize=None)
     def _shard_version(cls, path: str):
-        file_path = os.path.join(path, "shard.yml")
+        file_path = _find_above(path, "shard.yml")
+        with open(file_path, "rb") as f:
+            m = re.search(rb"^version: *([\S+]+)", f.read(), flags=re.MULTILINE)
+        if not m:
+            raise PluginError(f"`version:` not found in {file_path!r}")
+
+
+def _find_above(path: str, filename: str) -> str:
+    orig_path = path
+    while path:
+        file_path = os.path.join(path, filename)
         if os.path.isfile(file_path):
-            with open(file_path, "rb") as f:
-                m = re.search(rb"^version: *([\S+]+)", f.read(), flags=re.MULTILINE)
-            if not m:
-                raise PluginError(f"`version:` not found in {file_path!r}")
-            return m[1].decode()
-        if not path:
-            raise PluginError(f"'shard.yml' not found anywhere above {path!r}")
-        return cls._shard_version(os.path.dirname(path))
+            return file_path
+        path = os.path.dirname(path)
+    raise PluginError(f"{filename!r} not found anywhere above {os.path.abspath(orig_path)!r}")
+
+
+class _CrystalInfo:
+    @cached_property
+    def crystal_version(self) -> str:
+        return subprocess.check_output(
+            ["crystal", "env", "CRYSTAL_VERSION"], encoding="ascii"
+        ).rstrip()
+
+    @cached_property
+    def crystal_src(self):
+        out = subprocess.check_output(["crystal", "env", "CRYSTAL_PATH"], text=True).rstrip()
+        for path in out.split(os.pathsep):
+            if os.path.isfile(os.path.join(path, "prelude.cr")):
+                return os.path.relpath(path)
+        raise PluginError(f"Crystal sources not found anywhere in CRYSTAL_PATH={out!r}")
+
+
+class _DictAccess:
+    def __init__(self, obj):
+        self.obj = obj
+
+    def __getitem__(self, key):
+        try:
+            return getattr(self.obj, key)
+        except AttributeError as e:
+            raise KeyError(f"Missing key: {e}")
+
+
+_crystal_info = _DictAccess(_CrystalInfo())
 
 
 class DocRoot(DocModule):
@@ -185,7 +190,7 @@ class DocView:
         return val
 
     def walk_types(self) -> Iterator[DocType]:
-        types = self.types  # type: DocMapping[DocType]
+        types = cast(DocMapping[DocType], self.types)
         for typ in types:
             yield typ
             yield from typ.walk_types()
