@@ -6,7 +6,10 @@ import dataclasses
 import re
 from typing import Any, Generic, Iterator, Mapping, Optional, Sequence, TypeVar, Union
 
-from cached_property import cached_property
+try:
+    from functools import cached_property
+except ImportError:
+    from cached_property import cached_property
 from mkdocstrings.handlers.base import CollectionError
 
 from . import crystal_html
@@ -23,7 +26,7 @@ class DocItem(metaclass=abc.ABCMeta):
     def __init__(self, data: Mapping[str, Any], parent: Optional[DocItem], root: Optional[DocType]):
         self.data = data
         self.parent = parent
-        self.root = root or self
+        self.root = root or self  # type: ignore
 
     @property
     def name(self) -> str:
@@ -512,3 +515,11 @@ class DocPath:
 
     def __repr__(self) -> str:
         return repr(self.full_name)
+
+    def __eq__(self, other) -> bool:
+        if isinstance(other, DocPath):
+            other = other.abs_id
+        return isinstance(other, str) and self.abs_id == other
+
+    def __hash__(self):
+        return hash(self.abs_id)
